Search OCaml packages
max) { row = max } }" @keyup.up="if (row !== null) { row -=1; if (row < 0) { row = null } }" @keyup.right="if (col < 1) col++" @keyup.left="if (col >= 1) col--" :aria-activedescendant="row !== null ? 'package-autocomplete-'+row+'-'+col : null" hx-get="/packages/autocomplete" hx-params="q" hx-trigger="keyup changed, search" hx-target="#header-search-results" hx-indicator="#header-search-indicator" autocomplete="off" >
Standard Library API
Learn
Tools
Packages
Community
News
Play
Search OCaml packages
Learn
Tools
Packages
Community
News
Play
Standard Library API
Get started
package
cow
latest (2.5.0)
2.5.0
2.4.0
2.3.0
2.2.0
2.0.1
search
Overview
Docs
You can search for identifiers within the package.
in-package search v0.2.0
package cow
cow
CHANGES
LICENSE
README
Library
cow
Cow
Xml
Json
Markdown
Html
Xhtml
Atom
Cow_atom
Cow_html
Create
Tags
Cow_json
Cow_markdown
Cow_xhtml
Cow_xml
String
Buffer
S
Make
Sources
cow
cow.ml
cow_atom.ml
cow_html.ml
cow_json.ml
cow_markdown.ml
cow_xhtml.ml
cow_xml.ml
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Package index
cow
LICENSE
(* Copyright
*
Permission to use, copy, modify, and distribute this software for any
purpose with or without fee is hereby granted, provided that the above
copyright notice and this permission notice appear in all copies.
THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. *)