Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Page
Library
Module
Module type
Parameter
Class
Class type
CowCamel on the Web (CoW) library.
This library includes module aliases for various useful Web formats such as XML, XHTML, JSON, Markdown and Atom. In many cases, you can also use the constitutent libraries directly.
module Xml = Cow_xmlmodule Json = Cow_jsonmodule Markdown = Cow_markdownmodule Html = Cow_htmlmodule Xhtml = Cow_xhtmlmodule Atom = Cow_atom